import numpy as np
import pandas as pd
from os import mkdir
from cmlreaders import CMLReader, get_data_index
df = get_data_index("r1")

class cml_pipeline():
    
    def __init__(self, subject, experiment,):
        self.sub = subject
        self.exp = experiment
        
        
        #Get montage and localization info for this subject/experiment
        locs = df[(df['subject']==self.sub) & (df['experiment']==self.exp)]['localization'].unique()
        monts = df[(df['subject']==self.sub) & (df['experiment']==self.exp)]['montage'].unique()
        
        #Check if more than one montage/localization was used in this experiment.
        if len(locs)>1:
            print('More than one localization! Defaulting to first.')
            self.loc = locs[0]
        else:
            self.loc = locs[0]
        if len(monts)>1:
            print('More than one montage! Defaulting to first.')
            self.mont = monts[0]
        else:
            self.mont = monts[0]
        
        #Set available session numbers
        self.sessions = df[np.logical_and(df["subject"] == self.sub, df['experiment']==self.exp)]['session'].unique()
        self.reader = CMLReader(self.sub, self.exp, montage=self.mont, localization=self.loc)
        
    def set_events(self, sess=None):
        #Loads all the events for the given experiment. Optionally specify session. 
        
        if sess is None:
            self.evs = pd.concat([CMLReader(self.sub, self.exp, sess, montage=self.mont, localization=self.loc).load('events') for sess in self.sessions])
        else:
            reader = CMLReader(self.sub, self.exp, sess, montage=self.mont, localization=self.loc)
            self.evs = reader.load('task_events')
        return
            
    def set_elecs(self, type='bi', flag_bad_elecs=True):
        #Load electrode information. Use 'bi' for bipolar pairs or 'mono' for unrereferenced monopolars
        
        reader = self.reader
        if type=='bi':
            self.elecs = reader.load('pairs')
        elif type=='mono':
            self.elecs = reader.load('contacts')
        else:
            raise ValueError('Please specify either "bi" or "mono" for electrode type.')
        
        #For removing electrodes in electrode_categories.txt
        if flag_bad_elecs:
            try:
                elec_cat = reader.load('electrode_categories')
            except:
                print('No electrode categories file found!')
                return
            
            cat_names = np.array(['' for i in range(len(self.elecs))]).astype('U20')
            cat_bool = np.zeros(len(self.elecs)).astype(bool)
            
            #Find the contacts/pairs with labels in electrode_categories and mark as bad
            if type=='mono':
                for k, v in elec_cat.items():
                    for l in v:
                        idxs = np.where(self.elecs['label']==l)
                        cat_bool[idxs] = True
                        cat_names[idxs] = k
                        
            if type=='bi':
                for k, v in elec_cat.items():
                    for l in v:
                        an_idx = self.elecs['label'].str.split('-').str.get(0).str.contains(l)
                        cat_idx =  self.elecs['label'].str.split('-').str.get(1).str.contains(l)
                        cat_bool[np.where(np.logical_or(an_idx, cat_idx))] = True
                        cat_names[np.where(np.logical_or(an_idx, cat_idx))] = k
                        
            #Add new columns to existing elecs dataframe
            self.elecs['bad_elecs'] = pd.Series(data=cat_bool, index=self.elecs.index)
            self.elecs['electrode_categories'] = pd.Series(data=cat_names, index=self.elecs.index)
            
    def get_resting_events(self):
        from copy import copy
        #Use the FR countdown period to establish baseline events, spaced every second
        
        #Get samplerate to do eegoffsets -- note that we default to the sr for the initial session!
        reader = CMLReader(self.sub, self.exp, montage=self.mont, localization=self.loc, session=self.sessions[0])
        init_sr = int(reader.load("sources")['sample_rate'])
        self.sr = init_sr
        
        rest_evs = self.evs[self.evs['type']=='COUNTDOWN_START']
        for i in range(1, 10): #for each second in the 10-second countdown interval
            rest_copy = copy(self.evs[self.evs['type']=='COUNTDOWN_START'])
            offsets = np.array(rest_copy['eegoffset'])+int(init_sr*i)
            rest_copy['eegoffset'] = pd.Series(data=offsets, index=rest_copy.index)
            rest_evs = pd.concat([rest_evs, rest_copy])
            
        return rest_evs
    
    def get_resting_connectivity(self, evs, fmin, fmax, method='coh', mode='multitaper'):
        #Use MNE to get connectivity for resting events extracted using get_resting_events method.
        
        reader = self.reader
        try:  #some biplar ENS subjects will have nonmatching EEG and pairs information
            eeg = reader.load_eeg(events=evs, rel_start=0, rel_stop=1000, scheme=self.elecs)  #Load 1-second clips of EEG
            self.eeg = eeg.to_mne()
        except:
            eeg = reader.load_eeg(events=evs, rel_start=0, rel_stop=1000)
            self.eeg = eeg.to_mne()
            if len(self.elecs)!=eeg.shape[1]:
                raise ValueError('pairs.json and loaded EEG do not match! Probably should not use.')
            
        from mne.connectivity import spectral_connectivity
        cons, freqs, times, n_epochs, n_tapers = spectral_connectivity(
            self.eeg, method=method, mode=mode, sfreq=self.sr, fmin=fmin, fmax=fmax,
            faverage=True, tmin=0.0, mt_adaptive=False, n_jobs=1, verbose=False,)
                
        #Symmetrize and save average network
        mu = np.mean(cons, 2)
        mu_full = np.nansum(np.array([mu, mu.T]), 0)
        mu_full[np.diag_indices_from(mu_full)] = 0.0
        #np.save(self.root+''+self.s+'/'+self.s+'_baseline10trials_network_'+str(self.band)+'.npy', mu_full)
        
        return mu_full
        
            
            
        
        
            